@@ -1748,6 +1748,8 @@ START_TEST(touchpad_tap_default_disabled)
 
 	/* this test is only run on specific devices */
 
+	ck_assert_int_eq(libinput_device_config_tap_get_enabled(dev->libinput_device),
+			 LIBINPUT_CONFIG_TAP_DISABLED);
 	ck_assert_int_eq(libinput_device_config_tap_get_default_enabled(dev->libinput_device),
 			 LIBINPUT_CONFIG_TAP_DISABLED);
 }
@@ -1759,6 +1761,8 @@ START_TEST(touchpad_tap_default_enabled)
 
 	/* this test is only run on specific devices */
 
+	ck_assert_int_eq(libinput_device_config_tap_get_enabled(dev->libinput_device),
+			 LIBINPUT_CONFIG_TAP_ENABLED);
 	ck_assert_int_eq(libinput_device_config_tap_get_default_enabled(dev->libinput_device),
 			 LIBINPUT_CONFIG_TAP_ENABLED);
 }
@@ -1788,6 +1792,26 @@ START_TEST(touchpad_tap_default_map)
 }
 END_TEST
 
+START_TEST(touchpad_tap_map_unsupported)
+{
+	struct litest_device *dev = litest_current_device();
+	enum libinput_config_tap_button_map map;
+	enum libinput_config_status status;
+
+	map = libinput_device_config_tap_get_button_map(dev->libinput_device);
+	ck_assert_int_eq(map, LIBINPUT_CONFIG_TAP_MAP_LRM);
+	map = libinput_device_config_tap_get_default_button_map(dev->libinput_device);
+	ck_assert_int_eq(map, LIBINPUT_CONFIG_TAP_MAP_LRM);
+
+	status = libinput_device_config_tap_set_button_map(dev->libinput_device,
+							   LIBINPUT_CONFIG_TAP_MAP_LMR);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_UNSUPPORTED);
+	status = libinput_device_config_tap_set_button_map(dev->libinput_device,
+							   LIBINPUT_CONFIG_TAP_MAP_LRM);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_UNSUPPORTED);
+}
+END_TEST
+
 START_TEST(touchpad_tap_set_map)
 {
 	struct litest_device *dev = litest_current_device();
@@ -1842,6 +1866,20 @@ START_TEST(touchpad_tap_set_map_no_tapping)
 }
 END_TEST
 
+START_TEST(touchpad_tap_get_map_no_tapping)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	enum libinput_config_tap_button_map map;
+
+	map = libinput_device_config_tap_get_button_map(device);
+	ck_assert_int_eq(map,  LIBINPUT_CONFIG_TAP_MAP_LRM);
+
+	map = libinput_device_config_tap_get_default_button_map(device);
+	ck_assert_int_eq(map,  LIBINPUT_CONFIG_TAP_MAP_LRM);
+}
+END_TEST
+
 START_TEST(touchpad_tap_map_delayed)
 {
 	struct litest_device *dev = litest_current_device();
@@ -1886,6 +1924,8 @@ START_TEST(touchpad_drag_default_disabled)
 
 	ck_assert_int_eq(libinput_device_config_tap_get_default_drag_enabled(dev->libinput_device),
 			 LIBINPUT_CONFIG_DRAG_DISABLED);
+	ck_assert_int_eq(libinput_device_config_tap_get_drag_enabled(dev->libinput_device),
+			 LIBINPUT_CONFIG_DRAG_DISABLED);
 }
 END_TEST
 
@@ -1897,6 +1937,8 @@ START_TEST(touchpad_drag_default_enabled)
 
 	ck_assert_int_eq(libinput_device_config_tap_get_default_drag_enabled(dev->libinput_device),
 			 LIBINPUT_CONFIG_DRAG_ENABLED);
+	ck_assert_int_eq(libinput_device_config_tap_get_drag_enabled(dev->libinput_device),
+			 LIBINPUT_CONFIG_DRAG_ENABLED);
 }
 END_TEST
 
@@ -1911,6 +1953,24 @@ START_TEST(touchpad_drag_config_invalid)
 }
 END_TEST
 
+START_TEST(touchpad_drag_config_unsupported)
+{
+	struct litest_device *dev = litest_current_device();
+	enum libinput_config_status status;
+
+	ck_assert_int_eq(libinput_device_config_tap_get_default_drag_enabled(dev->libinput_device),
+			 LIBINPUT_CONFIG_DRAG_DISABLED);
+	ck_assert_int_eq(libinput_device_config_tap_get_drag_enabled(dev->libinput_device),
+			 LIBINPUT_CONFIG_DRAG_DISABLED);
+	status = libinput_device_config_tap_set_drag_enabled(dev->libinput_device,
+							     LIBINPUT_CONFIG_DRAG_ENABLED);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_UNSUPPORTED);
+	status = libinput_device_config_tap_set_drag_enabled(dev->libinput_device,
+							     LIBINPUT_CONFIG_DRAG_DISABLED);
+	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_SUCCESS);
+}
+END_TEST
+
 START_TEST(touchpad_drag_config_enabledisable)
 {
 	struct litest_device *dev = litest_current_device();
@@ -2171,8 +2231,10 @@ litest_setup_tests_touchpad_tap(void)
 	litest_add("tap:config", touchpad_tap_is_not_available, LITEST_ANY, LITEST_TOUCHPAD);
 
 	litest_add("tap:config", touchpad_tap_default_map, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add("tap:config", touchpad_tap_map_unsupported, LITEST_ANY, LITEST_TOUCHPAD);
 	litest_add("tap:config", touchpad_tap_set_map, LITEST_TOUCHPAD, LITEST_ANY);
 	litest_add("tap:config", touchpad_tap_set_map_no_tapping, LITEST_ANY, LITEST_TOUCHPAD);
+	litest_add("tap:config", touchpad_tap_get_map_no_tapping, LITEST_ANY, LITEST_TOUCHPAD);
 	litest_add("tap:config", touchpad_tap_map_delayed, LITEST_TOUCHPAD, LITEST_SINGLE_TOUCH|LITEST_SEMI_MT);
 
 	litest_add("tap-1fg:1fg", clickpad_1fg_tap_click, LITEST_CLICKPAD, LITEST_ANY);
@@ -2184,6 +2246,7 @@ litest_setup_tests_touchpad_tap(void)
 	litest_add("tap:drag", touchpad_drag_default_disabled, LITEST_ANY, LITEST_TOUCHPAD);
 	litest_add("tap:drag", touchpad_drag_default_enabled, LITEST_TOUCHPAD, LITEST_BUTTON);
 	litest_add("tap:drag", touchpad_drag_config_invalid, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add("tap:drag", touchpad_drag_config_unsupported, LITEST_ANY, LITEST_TOUCHPAD);
 	litest_add("tap:drag", touchpad_drag_config_enabledisable, LITEST_TOUCHPAD, LITEST_ANY);
 	litest_add("tap:drag", touchpad_drag_disabled, LITEST_TOUCHPAD, LITEST_ANY);
 	litest_add("tap:drag", touchpad_drag_disabled_immediate, LITEST_TOUCHPAD, LITEST_ANY);
