@@ -63,6 +63,7 @@ enum options {
 	OPT_SPEED,
 	OPT_PROFILE,
 	OPT_SHOW_KEYCODES,
+	OPT_QUIET,
 };
 
 LIBINPUT_ATTRIBUTE_PRINTF(3, 0)
@@ -72,7 +73,25 @@ log_handler(struct libinput *li,
 	    const char *format,
 	    va_list args)
 {
+#define ANSI_HIGHLIGHT "\x1B[0;1;39m"
+#define ANSI_RED "\x1B[0;31m"
+#define ANSI_NORMAL "\x1B[0m"
+	static int is_tty = -1;
+
+	if (is_tty == -1)
+		is_tty = isatty(STDOUT_FILENO);
+
+	if (is_tty) {
+		if (priority >= LIBINPUT_LOG_PRIORITY_ERROR)
+			printf(ANSI_RED);
+		else if (priority >= LIBINPUT_LOG_PRIORITY_INFO)
+			printf(ANSI_HIGHLIGHT);
+	}
+
 	vprintf(format, args);
+
+	if (is_tty && priority >= LIBINPUT_LOG_PRIORITY_INFO)
+		printf(ANSI_NORMAL);
 }
 
 void
@@ -112,6 +131,7 @@ tools_usage(void)
 	       "Other options:\n"
 	       "--grab .......... Exclusively grab all openend devices\n"
 	       "--verbose ....... Print debugging output.\n"
+	       "--quiet ......... Only print libinput messages, useful in combination with --verbose.\n"
 	       "--help .......... Print this help.\n",
 		program_invocation_short_name);
 }
@@ -156,6 +176,7 @@ tools_parse_args(int argc, char **argv, struct tools_context *context)
 			{ "grab", 0, 0, OPT_GRAB },
 			{ "help", 0, 0, OPT_HELP },
 			{ "verbose", 0, 0, OPT_VERBOSE },
+			{ "quiet", 0, 0, OPT_QUIET },
 			{ "enable-tap", 0, 0, OPT_TAP_ENABLE },
 			{ "disable-tap", 0, 0, OPT_TAP_DISABLE },
 			{ "enable-drag", 0, 0, OPT_DRAG_ENABLE },
@@ -344,6 +365,9 @@ tools_parse_args(int argc, char **argv, struct tools_context *context)
 		case OPT_SHOW_KEYCODES:
 			options->show_keycodes = true;
 			break;
+		case OPT_QUIET:
+			options->quiet = true;
+			break;
 		default:
 			tools_usage();
 			return 1;
